sara

Values

pub fn elapsed(during fun: fn() -> a) -> #(a, Float)
pub fn main() -> Nil

🦑 Performs serialization code generation for your Gleam project.

The sara cli is not configurable by default. With the JSON module of Sara you can with the help of //@json_encode() and //@json_decode() generates type-safe JSON encoding and decoding functions for your Gleam custom types at build time.

⚠️ The generated JSON code relies on :

  • json package for the json encoder.
  • stdlib package for the json decoder. So make sure to have them inside your project.
Search Document