sara
Values
pub fn main() -> Nil
🦑 Performs serialization code generation for your Gleam project.
The sara cli is not configurable by default.
With the JSON module of Sara you can with the help of //@json_encode() and //@json_decode() generates type-safe JSON encoding and decoding functions for your Gleam custom types at build time.
⚠️ The generated JSON code relies on :